import gleam/dynamic/decode.{type Decoder}
import gleam/option.{type Option}
import models/faction_symbol.{type FactionSymbol}
import models/faction_trait.{type FactionTrait}
import utils/decode as decode_utils

pub type Faction {
  Faction(
    symbol: FactionSymbol,
    name: String,
    description: String,
    headquarters: Option(String),
    traits: List(FactionTrait),
    is_recruiting: Bool,
  )
}

pub fn decoder() -> Decoder(Faction) {
  use symbol <- decode.field("symbol", faction_symbol.decoder())
  use name <- decode.field("name", decode.string)
  use description <- decode.field("description", decode.string)
  use headquarters <- decode_utils.field_key_value_optional(
    "headquarters",
    decode.string,
  )
  use traits <- decode.field("traits", decode.list(faction_trait.decoder()))
  use is_recruiting <- decode.field("isRecruiting", decode.bool)
  decode.success(Faction(
    symbol:,
    name:,
    description:,
    headquarters:,
    traits:,
    is_recruiting:,
  ))
}
