import gleam/dynamic/decode.{type Decoder}
import gleam/option.{type Option}
import models/mount_deposit.{type MountDeposit}
import models/mount_symbol.{type MountSymbol}
import models/ship_requirements.{type ShipRequirements}
import utils/decode as decode_utils

pub type ShipMount {
  ShipMount(
    symbol: MountSymbol,
    name: String,
    description: String,
    strength: Option(Int),
    deposits: Option(List(MountDeposit)),
    requirements: ShipRequirements,
  )
}

pub fn decoder() -> Decoder(ShipMount) {
  use symbol <- decode.field("symbol", mount_symbol.decoder())
  use name <- decode.field("name", decode.string)
  use description <- decode.field("description", decode.string)
  use strength <- decode_utils.field_key_value_optional("strength", decode.int)
  use deposits <- decode_utils.field_key_value_optional(
    "deposits",
    decode.list(mount_deposit.decoder()),
  )
  use requirements <- decode.field("requirements", ship_requirements.decoder())
  decode.success(ShipMount(
    symbol:,
    name:,
    description:,
    strength:,
    deposits:,
    requirements:,
  ))
}
