import gleam/dynamic/decode.{type Decoder}
import utils/decode as decode_utils

pub type SystemType {
  NeutronStar
  RedStar
  OrangeStar
  BlueStar
  YoungStar
  WhiteDwarf
  BlackHole
  Hypergiant
  Nebula
  Unstable
}

pub fn to_string(system_type: SystemType) -> String {
  case system_type {
    NeutronStar -> "NEUTRON_STAR"
    RedStar -> "RED_STAR"
    OrangeStar -> "ORANGE_STAR"
    BlueStar -> "BLUE_STAR"
    YoungStar -> "YOUNG_STAR"
    WhiteDwarf -> "WHITE_DWARF"
    BlackHole -> "BLACK_HOLE"
    Hypergiant -> "HYPERGIANT"
    Nebula -> "NEBULA"
    Unstable -> "UNSTABLE"
  }
}

pub fn parse(system_type_str: String) -> Result(SystemType, SystemType) {
  case system_type_str {
    "NEUTRON_STAR" -> Ok(NeutronStar)
    "RED_STAR" -> Ok(RedStar)
    "ORANGE_STAR" -> Ok(OrangeStar)
    "BLUE_STAR" -> Ok(BlueStar)
    "YOUNG_STAR" -> Ok(YoungStar)
    "WHITE_DWARF" -> Ok(WhiteDwarf)
    "BLACK_HOLE" -> Ok(BlackHole)
    "HYPERGIANT" -> Ok(Hypergiant)
    "NEBULA" -> Ok(Nebula)
    "UNSTABLE" -> Ok(Unstable)
    _ -> Ok(NeutronStar)
  }
}

pub fn decoder() -> Decoder(SystemType) {
  decode_utils.enum_decoder(parse, "SystemType")
}
